import ts from "typescript";
const isTTY = process.env.TTY || process.stdout.isTTY;
export class BaseError extends Error {
    diagnostic;
    constructor(diagnostic) {
        super(ts.flattenDiagnosticMessageText(diagnostic.messageText, "\n"));
        this.diagnostic = BaseError.createDiagnostic(diagnostic);
    }
    static createDiagnostic(diagnostic) {
        if (diagnostic.node && diagnostic.node.pos !== -1) {
            diagnostic.file = diagnostic.node.getSourceFile();
            diagnostic.start = diagnostic.node.getStart();
            diagnostic.length = diagnostic.node.getWidth();
            diagnostic.node = undefined;
        }
        diagnostic.code = `J - ${diagnostic.code}`;
        return Object.assign({
            category: ts.DiagnosticCategory.Error,
            file: undefined,
            length: 0,
            start: 0,
        }, diagnostic);
    }
    format() {
        const formatter = isTTY ? ts.formatDiagnosticsWithColorAndContext : ts.formatDiagnostics;
        return formatter([this.diagnostic], {
            getCanonicalFileName: (fileName) => fileName,
            getCurrentDirectory: () => "",
            getNewLine: () => "\n",
        });
    }
}
//# sourceMappingURL=BaseError.js.map