import ts from "typescript";
import { BaseError } from "./BaseError.js";
export class UnknownNodeError extends BaseError {
    node;
    constructor(node) {
        super({
            code: 100,
            node,
            messageText: `Unknown node of kind "${ts.SyntaxKind[node.kind]}"`,
        });
        this.node = node;
    }
}
export class UnknownTypeError extends BaseError {
    type;
    constructor(type) {
        super({
            code: 101,
            messageText: `Unknown type "${type.getId()}"`,
        });
        this.type = type;
    }
}
export class RootlessError extends BaseError {
    fullName;
    constructor(fullName) {
        super({
            code: 102,
            messageText: `No root type "${fullName}" found`,
        });
        this.fullName = fullName;
    }
}
export class MultipleDefinitionsError extends BaseError {
    name;
    defA;
    defB;
    constructor(name, defA, defB) {
        super({
            code: 103,
            messageText: `Type "${name}" has multiple definitions.`,
        });
        this.name = name;
        this.defA = defA;
        this.defB = defB;
    }
}
export class LogicError extends BaseError {
    node;
    constructor(node, messageText) {
        super({
            code: 104,
            messageText,
            node,
        });
        this.node = node;
    }
}
export class ExpectationFailedError extends BaseError {
    node;
    constructor(messageText, node) {
        super({
            code: 105,
            messageText,
            node,
        });
        this.node = node;
    }
}
export class JsonTypeError extends BaseError {
    type;
    constructor(messageText, type) {
        super({
            code: 106,
            messageText,
        });
        this.type = type;
    }
}
export class DefinitionError extends BaseError {
    definition;
    constructor(messageText, definition) {
        super({
            code: 107,
            messageText,
        });
        this.definition = definition;
    }
}
export class BuildError extends BaseError {
    constructor(diag) {
        super({
            code: 108,
            ...diag,
        });
    }
}
//# sourceMappingURL=Errors.js.map