import ts from "typescript";
import { ExpectationFailedError } from "../Error/Errors.js";
import { LiteralType } from "../Type/LiteralType.js";
export class PrefixUnaryExpressionNodeParser {
    childNodeParser;
    constructor(childNodeParser) {
        this.childNodeParser = childNodeParser;
    }
    supportsNode(node) {
        return node.kind === ts.SyntaxKind.PrefixUnaryExpression;
    }
    createType(node, context) {
        const operand = this.childNodeParser.createType(node.operand, context);
        if (operand instanceof LiteralType) {
            switch (node.operator) {
                case ts.SyntaxKind.PlusToken:
                    return new LiteralType(+operand.getValue());
                case ts.SyntaxKind.MinusToken:
                    return new LiteralType(-operand.getValue());
                case ts.SyntaxKind.TildeToken:
                    return new LiteralType(~operand.getValue());
                case ts.SyntaxKind.ExclamationToken:
                    return new LiteralType(!operand.getValue());
            }
            throw new ExpectationFailedError("Unsupported prefix unary operator", node);
        }
        throw new ExpectationFailedError(`Expected operand to be "LiteralType" but is "${operand ? operand.constructor.name : operand}"`, node);
    }
}
//# sourceMappingURL=PrefixUnaryExpressionNodeParser.js.map